/*
*******************************************************************************
** Copyright (C) 1995 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
******************************************************************************
*/

/*
------------------------------------------------------------------------------

File Name:      bctfp.app.h

Description:
        User includable header for driver interface.

Modification History:
        Who     Date            What
        ---     ----            ----
        bpw     08/28/95        created

-------------------------------------------------------------------------------
*/

/*
 * Include files
 */
#include "fcntl.h"
#include "unistd.h"
#include "ioLib.h"
#include "stdDef.h"

#ifndef __BCTFP_APP_H__
#define __BCTFP_APP_H__

/*
 * Defines
 */
/* parameter to this macro must be of type BCDStruct */
#define SECS_TO_SINGLE(x) (SINGLE)((SINGLE)x.secs+((SINGLE)x.esec*1e-7))

/*
 * typedefs
 */
/*
  The following enum specifies all the allowable IOCTL calls.  For
  DEBUG, PACKET, and SIGNAL, they can be turned on or off by using
  an ioctl call like this:

  ioctl(fd,DEBUG,ON);  <-- turns on driver DEBUG mode
  ioctl(fd,PACKET,OFF); <-- turns off driver PACKET mode

  For the READREG command the ioctl will look like this:

  unsigned short val = R_INTSTAT;
  ioctl(fd,READREG,&val); <-- should read the INTSTAT board register

  All registers can be read as follows:
  BTFPRegs vals;
  vals.btfpId = R_ALL;
  ioctl(fd,READREG,&vals); <-- should read all board registers

  The driver will automatically determine what type of data should
  have been passed to it based on the value of the argument.  Therefore
  it would be an error to do the following:

  int vals=R_ALL;
  ioctl(fd,READREG,&vals); <-- will fill in unallocated memory
  */
typedef
enum _ioctls
{
    INTCONN     = 0x00,         /* Connect the user specified function to
                                 * driver interrupt.  This is effectively the
                                 * same as connecting the interrupt to
                                 * SIGUSR1.  The driver interrupt will call
                                 * the specified function with the INTSTAT
                                 * register as a parameter.  This will allow
                                 * the user to multiplex their interrupt
                                 * code. NOTE 1: Any function connected in
                                 * this way must follow vxWorks interrupt
                                 * code specs. NOTE 2: All interrupt
                                 * processing done by the driver will be
                                 * completed at the time of this call.
                                 * Therefore, all structures normally filled
                                 * in by this interrupt can be expected to be
                                 * correct during the user interrupt call */
    DEBUG       = 0x01,         /* Set driver  DEBUG mode */
    PACKET      = 0x02,         /* Set driver PACKET mode */
    SIGNAL      = 0x03,         /* Set driver SIGNAL mode */
    ROUTFIFO    = 0x04,         /* Read one byte from in fifo */
    WINFIFO     = 0x05,         /* write one byte to out fifo */
    READREG     = 0x06,         /* Read specified register or all if 0xFF */
    WRITEREG    = 0x07,         /* Write specified register or all if 0xFF */
    READTIME    = 0x08,         /* Read time from time regs */
    READEVENT   = 0x09,         /* Read event regs */
    WRITESTROBE = 0x0A,         /* Write strobe regs */
    LATCHEVENT  = 0x0B,         /* Latch event registers */
    LATCHTIME   = 0x0C          /* Latch time registers */
      /* Other ioctls contained in Packets enum */
}       Ioctls;

/*
  - The following enums are used instead of defines for two reasons:
  - 1) Debuggers know about enums and they do not know about #defines
  -    so making something that would be a define otherwise makes it
  -    easier to debug  (Name of value instead of having to find number).
  - 2) enums are cast as ints.  This at least gives them a type which
  -    might assist in debugging as well.
  */

typedef
enum _RegOffs
{
    R_ID = 0x00,                /* ID Register */
    R_DEVICE = 0x02,            /* VXI Device Register */
    R_STATUS = 0x04,            /* VXI Status Register */
    R_CONTROL = 0x04,           /* Control Register */
    R_TIMEREQ = 0x0A,           /* Time Request Register */
    R_TIME0 = 0x0C,             /* Time 0 (days hundreds) register */
    R_TIME1 = 0x0E,             /* Time 1 (days tens, units - hours tens,
                                 * units) */
    R_TIME2 = 0x10,             /* Time 2 (minutes tens, units - seconds
                                 * tens, units) */
    R_TIME3 = 0x12,             /* Time 3 (seconds 10^-1 to 10^-4) */
    R_TIME4 = 0x14,             /* Time 4 (seconds 10^-5 to 10^-7) */
    R_EVENT0 = 0x16,            /* Event 0 */
    R_EVENT1 = 0x18,            /* Event 1 */
    R_EVENT2 = 0x1A,            /* Event 2 */
    R_EVENT3 = 0x1C,            /* Event 3 */
    R_EVENT4 = 0x1E,            /* Event 4 */
    R_STROBE1 = 0x18,           /* Strobe 1 (Also Event 0) */
    R_STROBE2 = 0x1A,           /* Strobe 2 (Also Event 1) */
    R_STROBE3 = 0x1C,           /* Strobe 3 (Also Event 2) */
    R_UNLOCK = 0x20,            /* Unlock Register */
    R_ACK = 0x22,               /* Packet ACK Register */
    R_CMD = 0x24,               /* Command Register (TFP functions) */
    R_FIFO = 0x26,              /* Fifo Register (input/output) */
    R_MASK = 0x28,              /* Interrupt Mask Register */
    R_INTSTAT = 0x2A,           /* Interrupt Status Register */
    R_VECTOR = 0x2C,            /* Interrupt Vector Register */
    R_LEVEL  = 0x2E,            /* Interrupt Level Register */
    R_ALL = 0xFF                /* ALL REGS (for ioctl) */
} RegOffs;

typedef
enum _Packets
{
    P_MODE_SEL = 'A',           /* TFP Mode select */
    P_MJTM_SET = 'B',           /* Set Major Time */
    P_CMD_INP = 'C',            /* Command Input */
    P_DAC_LD = 'D',             /* Load D/A Converter */
    P_HBT_CTL = 'F',            /* Heartbeat control */
    P_OFF_CTL = 'G',            /* Offset Control */
    P_TC_FMAT = 'H',            /* Set Timecode format (mode 0) */
    P_SEL_CLK = 'I',            /* Clock Source Select */
    P_GPS_DATA = 'J',           /* GPS Receiver Data */
    P_SEL_GCODE = 'K',          /* Select generator code */
    P_SET_RCLK = 'L',           /* Set Real time clock */
    P_TIME_OFF = 'M',           /* Local Time offset select */
    P_DATA_REQ = 'O',           /* Request data from the TFP */
    P_PATH_SEL = 'P',           /* Path selection */
    P_SET_GAIN = 'Q',           /* Set Disciplining gain */
    P_SET_YEAR = 'S'            /* Set year */
} Packets;

typedef
struct _BTFPioctlData
{
    enum
    {
    TIME = 0, EVENT = 1, STROBE = 2} dataType;
    unsigned short btfpData[5];
}       BTFPioctlData;

typedef
enum _PcktHeader
{
    SOH = 0x01,
    DLE = 0x10,
    ETB = 0x17,
    ETX = 0x03
} PcktHeader;

typedef
enum _Bool
{
    ON = 0x01,
    OFF = 0x00
} Bool;


typedef
enum _masks
{
    TIME_REF = 0x0000,
    EVENT_REF = 0x0000,

    TIME_FLY = 0x0010,
    EVENT_FLY = 0x0010,

    TIME_OSCIN = 0x0000,
    EVENT_OSCIN = 0x0000,
    TIME_OSCOUT = 0x0040,
    EVENT_OSCOUT = 0x0040,
    /* Acknowledge masks */
    ACK_INFIFO = 0x0001,
    ACK_1PPS = 0x0002,
    ACK_DPA = 0x0004,
    ACK_MORE = 0x0010,
    ACK_CLEAR = 0x0010,
    ACK_INACT = 0x0080,
    /* Mode Enables */
    CMD_DISABLE = 0x0000,
    LOCK_EN = 0x0001,
    HBEAT_EN = 0x0002,
    EVSENSE_FALL = 0x0004,
    EVSENSE_RISE = 0x0000,
    EVENT_EN = 0x0008,
    STROBE_EN = 0x0010,
    STRMDE_MIN = 0x0020,
    STRMDE_MAJ = 0x0000,
    FREQSEL_10 = 0x0000,
    FREQSEL_05 = 0x0040,
    FREQSEL_01 = 0x0080,
    /* Interrupt Masks (INTMASK and INTSTAT) */
    INT_EXT = 0x0001,
    INT_HBEAT = 0x0002,
    INT_STROBE = 0x0004,
    INT_1PPS = 0x0008,
    INT_DPA = 0x0010,
    INT_DISABLE = 0x0000
} masks;

typedef
enum _modes
{
    /* Packet A modes */
    A_TIMECODE_DEC = '0',
    A_FREE_RUNNING = '1',
    A_EXT_1PPS = '2',
    A_REAL_TIME_CLK = '3',
    A_DIGITAL_SYNC = '4',
    A_GPS_ONBOARD = '5',
    A_GPS_ANTENNA = '6',
    A_DIAGNOSTIC = '7',
    /* Packet C modes */
    C_SOFT_RESET = '2',
    C_JAMSYNC = '3',
    C_BUF_RTC = '5',
    /* Packet H modes */
    H_IRIGA = 'A',
    H_IRIGB = 'B',
    H_2137 = 'C',
    H_NASA36 = 'N',
    H_XR3 = 'X',
    /* Packet I modes */
    I_EXT = 'E',
    I_INT = 'I',
    /* Packet K modes */
    K_IRIGB = 'B',
    K_IRIGH = 'H',
    /* Packet O modes */
    O_FMT0 = '0',
    O_FMT1 = '1',
    O_FMT2 = '2',
    O_FMT3 = '3',
    O_FMT4 = '4'
} modes;

typedef
struct _args
{
    INT     reg_id;
    UINT16 reg_val;
}       args;

typedef
struct _BCDStruct {
  INT
    days,
    hour,
    mins,
    secs,
    esec;
  UINT16
    status;
} BCDStruct;
typedef
void    (*FPTR) (INT);

/*
 * Imports
 */

/*
 * Exports
 */

/*
 * Locals
 */

/*
 * Forward declarations
 */
STATUS  btfpDevCreate(char *, void *, unsigned short, unsigned short);
#endif                          /* __BCTFP_APP_H__ */
